unit LD48GameUnit;

interface

Uses
  SysUtils, Classes,
  LD48MonsterUnit,
  S2DLLogger,
  SDL,
  S2DLImage,
  S2DLDraws,
  S2DLText;

type  TS2DLGameState = (gsNone, gsTitleProcess, gsHelp, gsGameSetup,
                        gsGameProcess, gsPlayerDied, gsLevelUp, gsLevelSetup,
                        gsGameOver, gsEndProcess, gsWinning, gsFinish);

Const
  LightChange = 5000;
  LightPickup = 850;

Type
  TS2DLGame = Class
  private
    FHelpPage: Integer;
    FGameState: TS2DLGameState;
    procedure SetHelpPage(const Value: Integer);published
    procedure SetGameState(const Value: TS2DLGameState);
  Private
    Event: TSDL_Event;   // Global Event Variable
    AppPath : String;    // Stores the Path of the Application to make loading easier
    Background: TS2DLImage; // Background Image of the Screen - save time by not requireing a redraw
                            // Of all tiles each frame (Can be extended to other uses)
    LastTick : Cardinal;
    Map : TMap;
    Player : TPlayer;
    Level : Integer;
    MapName : String;
    Property HelpPage : Integer read FHelpPage write SetHelpPage;
    Property GameState: TS2DLGameState read FGameState write SetGameState; // Current GameState
    Procedure CreateBackground;
    Procedure CreateTitleBackGround;
    Procedure CreateWinningBackGround;
    Procedure CreateHelpBackGround;
    Procedure CreateLevelUpBackground;
  Public
    Constructor Create;
    Destructor Destroy; Override;
    Procedure PlayerDied(Sender : TObject);
    Procedure LoadImages;
    Procedure LoadFonts;
    Procedure LoadSounds;
    Procedure LoadMap;
    procedure GameSetup;
    procedure RenderTitle;
    procedure RenderGame;
    Procedure RenderHelp;
    procedure RenderEndGame;
    procedure TitleProcess;
    Procedure HelpProcess;
    procedure LevelUpProcess;
    procedure PlayerDiedProcess;
    procedure GameOverProcess;
    Procedure LevelSetupProcess;
    procedure GameProcess;
    Procedure WinningProcess;
    procedure EndProcess;
    Procedure Run;
  End;

Var
  Game : TS2DLGame;

Implementation

Procedure TS2DLGame.CreateBackground;
Begin
  // Create a background to draw in each flip (in single image for performance)
  Images[0].DrawTiled(Background.ImageSurface,0,0,10,8,11);

End;

Procedure TS2DLGame.LoadImages;
Var
  Image : TS2DLImage;
Begin
  // Start Loading Images
{  Image := TS2DLImage.Create;
  Image.LoadFromFile(PChar(AppPath+'images\star.png'));
  Image.TransparentColor(0,0,0);
  Image.PatternWidth := 32;
  Image.PatternHeight := 32;
  Images.Add(Image);}
  Image := TS2DLImage.Create;
  Image.LoadFromFile(PChar(AppPath+'images\MapTile.png'));
  Image.PatternWidth := TileSize;
  Image.PatternHeight := TileSize;
  Images.Add(Image);
  Image := TS2DLImage.Create;
  Image.LoadFromFile(PChar(AppPath+'images\CircleOfLight.png'));
  Image.TransparentColor(255,255,255);
  Image.PatternWidth := CircleSize;
  Image.PatternHeight := CircleSize;
  Images.Add(Image);
  Image := TS2DLImage.Create;
  Image.LoadFromFile(PChar(AppPath+'images\DotEater.png'));
  Image.TransparentColor(255,255,255);
  Image.PatternWidth := PlayerSize;
  Image.PatternHeight := PlayerSize;
  Images.Add(Image);
  Image := TS2DLImage.Create;
  Image.LoadFromFile(PChar(AppPath+'images\FireDot.png'));
  Image.TransparentColor(255,255,255);
  Image.PatternWidth := FireSize;
  Image.PatternHeight := FireSize;
  Images.Add(Image);
  Image := TS2DLImage.Create;
  Image.LoadFromFile(PChar(AppPath+'images\Gems.png'));
  Image.TransparentColor(255,255,255);
  Image.PatternWidth := GemSize;
  Image.PatternHeight := GemSize;
  Images.Add(Image);
  Image := TS2DLImage.Create;
  Image.LoadFromFile(PChar(AppPath+'images\Monster.png'));
  //Image.Name := 'Monster';
  Image.TransparentColor(255,255,255);
  Image.PatternWidth := PlayerSize;
  Image.PatternHeight := PlayerSize;
  Images.Add(Image);
  Image := TS2DLImage.Create;
  Image.LoadFromFile(PChar(AppPath+'images\Man.png'));
  Image.Name := 'Man';
  Image.TransparentColor(140,133,127);
  Image.PatternWidth := 60;
  Image.PatternHeight := 60;
  Images.Add(Image);
  Image := TS2DLImage.Create;
    Image.LoadFromFile(PChar(AppPath+'images\Monster1.png'));
    Image.Name := 'Green'; Image.TransparentColor(140,133,127);
    Image.PatternWidth := 60; Image.PatternHeight := 60;
  Images.Add(Image);
  Image := TS2DLImage.Create;
    Image.LoadFromFile(PChar(AppPath+'images\Monster2.png'));
    Image.Name := 'Skeleton'; Image.TransparentColor(140,133,127);
    Image.PatternWidth := 60; Image.PatternHeight := 60;
  Images.Add(Image);
  Image := TS2DLImage.Create;
    Image.LoadFromFile(PChar(AppPath+'images\Monster3.png'));
    Image.Name := 'Purple'; Image.TransparentColor(140,133,127);
    Image.PatternWidth := 60; Image.PatternHeight := 60;
  Images.Add(Image);
  Image := TS2DLImage.Create;
    Image.LoadFromFile(PChar(AppPath+'images\Monster4.png'));
    Image.Name := 'Mummy'; Image.TransparentColor(140,133,127);
    Image.PatternWidth := 60; Image.PatternHeight := 60;
  Images.Add(Image);
  Image := TS2DLImage.Create;
    Image.LoadFromFile(PChar(AppPath+'images\Monster5.png'));
    Image.Name := 'Blue'; Image.TransparentColor(140,133,127);
    Image.PatternWidth := 60; Image.PatternHeight := 60;
  Images.Add(Image);
  Image := TS2DLImage.Create;
    Image.LoadFromFile(PChar(AppPath+'images\Monster6.png'));
    Image.Name := 'Lady'; Image.TransparentColor(140,133,127);
    Image.PatternWidth := 60; Image.PatternHeight := 60;
  Images.Add(Image);
  Image := TS2DLImage.Create;
    Image.LoadFromFile(PChar(AppPath+'images\ThemeDead.png'));
    Image.Name := 'Dead'; Image.TransparentColor(0,0,0);
  Images.Add(Image);
End;
Procedure TS2DLGame.LoadFonts;
Begin
  Log.LogStatus('Create Fonts','');
  Fonts.LoadFont('Images\24P_Arial_NeonYellow.png');
  Fonts.LoadFont('Images\24P_Copperplate_Blue.png');
  Fonts.LoadFont('Images\Small_Copperplate_Blue.PNG');
  Fonts.LoadFont('Images\24P_Copperplate_embosed.png');
  Log.LogStatus('Done Create Fonts','');

End;
Procedure TS2DLGame.LoadSounds;
Begin
{  Sounds.LoadSound('Beep','sounds\shot.wav');
  Sounds.LoadSound('Boop','sounds\shot.wav');
  Sounds.LoadSound('Ping','sounds\shot.wav');
  Sounds.LoadSound('GetReady','sounds\GetReady.wav');
  Sounds.LoadSound('Laugh','sounds\Laugh.wav');
  mmSystem.PlaySound(PChar('sounds\GetReady.wav'),0,SND_ASYNC); }
  LoadWav(PChar('sounds\GetReady.wav'),SndGetReady);
  LoadWav(PChar('sounds\Laugh.wav'),SndLaugh);
  LoadWav(PChar('sounds\Shot.wav'),SndShot);
End;

Constructor TS2DLGame.Create;
var
  Error: boolean;
  I : Integer;
begin
  AppPath := ExtractFilePath(ParamStr(0));
  Error := S2DLDraws.Error; // If there was an error in initializing S2DLDraw then dont do this
  if not Error then
  begin
    LoadImages;
    LoadFonts;
    LoadSounds;
    Background := TS2DLImage.CreateSizedSurface(640,480,24);
  end;

  if Error then
  begin
    // Free all images that were loaded
    // S2DLDraw Surface will automatically clear
    BackGround.Free;
    Halt(1);
  end;

  // Do any Program Initializations here
  Map := TMap.Create;
  MapName := 'Map';
  Map.OnDie := PlayerDied;
  Level := 1;
  For I := 1 to 4 do
    Keys[I] := False;
  Player := TPlayer.Create;
  Player.Map := Map;
  Map.Player := Player;
  Player.MoveSpeed := MoveSpeed;
  HelpPage := 1;
  GameState := gsTitleProcess;
end;

procedure TS2DLGame.GameSetup;
begin
  Log.LogError('Enter','GameSetup');
  // Do all Game Initilizations Here

  GameState := gsLevelUp;
  Map.LightCount := 1000;
  Map.LightBright := 0;
  Level := 1;
  Player.Score := 0;
  Player.Lives := 3;
  LoadMap;
  Map.Init;
end;

Destructor TS2DLGame.Destroy;
begin
  // End the Program
  Background.Free;
end;

procedure TS2DLGame.RenderTitle;
begin
  // Draw relevant Game Intro stuff. - Will be flipped in Title Process
  Background.Draw(0,0);

  Fonts[2].RenderFont(S2DLDraw.Surface,0,0,'FPS: '+IntToStr(S2DLDraw.FPS));
  Fonts[0].Write(60,420,'[ Help ]');
  Fonts[0].Write(270,420,'[ Play ]');
  Fonts[0].Write(480,420,'[ Quit ]');
end;

procedure TS2DLGame.RenderGame;
Var
  TC : Cardinal;
  IX,IY : Integer;
Begin
  TC := SDL_GetTicks - LastTick;
  Map.Move;
  Player.Move;
  Map.LightCount := Map.LightCount - TC;
  If Map.LightCount < 0 then
    Begin
      Map.LightBright := Map.LightBright + 1;
      If Map.LightBright > 4 then
        Map.LightBright := 4;
      Map.LightCount := Map.LightCount + LightChange;
    End;
  LastTick := SDL_GetTicks;
  If Map.Map[Player.MX,Player.MY].Dot = 2 then
    Begin
      Map.Map[Player.MX,Player.MY].Dot := 1;
      PlayTheSound(sndShot); //PlaySound('Ping');
      Map.TreasureCount := Map.TreasureCount - 1;
      Player.Score := Player.Score + 20;
      If Map.TreasureCount = 0 then
        Begin
          Level := Level + 1;
          MapName := Map.NextLevel;
          If MapName = 'End' then
            GameState := gsWinning
          Else
            Begin
              LoadMap;
              Map.Init;
              GameState := gsLevelUp;
            End;
        End;
    End;
  If Map.Map[Player.MX,Player.MY].Dot = 0 then
    Begin
      PlayTheSound(SndShot); //      PlaySound('Beep');
      Map.Map[Player.MX,Player.MY].Dot := 1;
      Map.DotCount := Map.DotCount - 1;
      Map.LightCount := Map.LightCount + LightPickup;
      If Map.LightCount > LightChange then
        If Map.LightBright > 0 then
          Begin
            Map.LightBright := Map.LightBright - 1;
            Map.LightCount := Map.LightCount - LightChange;
          End
        Else
          If Map.LightCount > 500 then
            Map.LightCount := 500;
    End;
  // Draw In-Game screen stuff. - Will be flipped in Game Process
//  Background.Draw((MapLeft div 64) * 64,(MapTop div 64) * 64);
  S2DLDraw.Clear;
  Map.Draw;
  Images[1].Draw(Player.DX-CircleSizeHalf-Map.MapLeft,Player.DY-CircleSizeHalf-Map.MapTop,Map.LightBright);
  For IX := -3 to 3 do
    For IY := -2 to 2 do
      If (IX <> 0) or (IY <> 0) then
      Begin
        Images[1].Draw((Player.DX+(CircleSize*IX))-CircleSizeHalf-Map.MapLeft,(Player.DY+(CircleSize*IY))-CircleSizeHalf-Map.MapTop,5);
      End;
  Fonts[2].RenderFont(S2DLDraw.Surface,0,0,'FPS: '+IntToStr(S2DLDraw.FPS));
  Fonts[0].RenderFont(S2DLDraw.Surface,10,50,'Fires Left: '+IntToStr(Map.DotCount));
  Fonts[0].RenderFont(S2DLDraw.Surface,10,80,'Treasures: '+IntToStr(Map.TreasureCount));
  Fonts[1].RenderFont(S2DLDraw.Surface,10,120,'Score: '+IntToStr(Player.Score));
  Fonts[1].RenderFont(S2DLDraw.Surface,10,150,'Lives: '+IntToStr(Player.Lives));
  Map.Draw2;

  If Player.Dead then
    Begin
      GameState := gsGameOver;
    End;
end;

procedure TS2DLGame.RenderEndGame;
begin
  // Draw End-Game screen stuff. - Will be flipped in Game Process
  Background.Draw(0,0);
  Fonts[0].Write(250,30,'Thanks');
  Fonts[1].Write(30,100,'My thanks to Ludum Dare for');
  Fonts[1].Write(30,130,'encouraging the 48 hour Game');
  Fonts[1].Write(30,160,'development contests.');
  Fonts[1].Write(30,220,'Thanks to Richard for making');
  Fonts[1].Write(30,250,'this contest happen and his');
  Fonts[1].Write(30,280,'effort on the web pages.');
end;

procedure TS2DLGame.TitleProcess;
Var
  X,Y : Integer;
begin
  { Draw title screen }
  repeat
    while SDL_PollEvent(@event) > 0 do
    begin
      case event.type_ of
        SDL_QUITEV :
           Begin
             Log.LogError('Quit Event','TitleProcess');
             GameState := gsFinish;
           End;
        SDL_MOUSEBUTTONUP:
        Begin
          X := Event.button.X;
          Y := Event.button.Y;
          If (Y > 415) and (Y < 455) then
            Begin
              If X < 230 then
                GameState := gsHelp;
              If (X < 440) and (X > 230) then
                GameState := gsGameSetup;
              If (X > 440) then
                GameState := gsEndProcess;
            End;
        End;

        SDL_KEYDOWN :
        begin
          case Event.Key.keysym.sym of
            SDLK_SPACE :
              GameState := gsGameSetup;
            SDLK_ESCAPE :
              Begin
                GameState := gsEndProcess;
              End;
          end;
        end;
        SDL_KEYUP :
        begin
          case Event.Key.keysym.sym of
            SDLK_F11 :
              GameState := gsWinning;
            SDLK_F1 :
              GameState := gsHelp;
          end;
        end;
      end;
    end;
    // Do screen animations and Game Logic for Title Screen here
    RenderTitle;

    S2DLDraw.Flip;
  until GameState <> gsTitleProcess;
end;

procedure TS2DLGame.GameProcess;
begin
  repeat
    while SDL_PollEvent(@event) > 0 do
    begin
      case event.type_ of
        SDL_QUITEV :
          Begin
            Log.LogError('Quit Event','GameProcess');
            GameState := gsFinish;
          End;

        SDL_KEYDOWN:
        begin
          case Event.Key.keysym.sym of
            SDLK_SPACE :
            begin
            end;

            SDLK_ESCAPE :
              Begin
                GameState := gsTitleProcess;
              End;
            SDLK_UP: Keys[1] := True;
            SDLK_Down: Keys[2] := True;
            SDLK_Left: Keys[3] := True;
            SDLK_Right: Keys[4] := True;
          end;
        end;
        SDL_KEYUP:
        begin
          case Event.Key.keysym.sym of
            SDLK_UP: Keys[1] := False;
            SDLK_Down: Keys[2] := False;
            SDLK_Left: Keys[3] := False;
            SDLK_Right: Keys[4] := False;
            SDLK_F10: Map.TreasureCount := 1;
          end;
        end;
      end;
    end;
    // Do game logic here

    RenderGame;
    S2DLDraw.Flip;
  until GameState <> gsGameProcess;
end;

procedure TS2DLGame.EndProcess;
begin
  repeat
    If SDL_GetTicks > LastTick + 3000 then
      Begin
        GameState := gsFinish;
      End;
    // Do any game over logic here - Usually Timer based
    RenderEndGame;
    S2DLDraw.Flip;
  until GameState <> gsEndProcess;
end;

Procedure TS2DLGame.Run;
begin
  Try
    randomize;
    GameState := gsTitleProcess;
    repeat
      case GameState of
        gsTitleProcess: TitleProcess;
        gsHelp: HelpProcess;
        gsGameSetup: GameSetup;
        gsLevelUp: LevelUpProcess;
        gsLevelSetup : LevelSetupProcess;
        gsGameProcess: GameProcess;
        gsGameOver: GameOverProcess;
        gsPlayerDied: PlayerDiedProcess;
        gsWinning: WinningProcess;
        gsEndProcess: EndProcess;
      end;
    until GameState = gsFinish;
  except
    log.logstatus('Error','End');
  End;
end;


procedure TS2DLGame.CreateTitleBackGround;
begin
  Images[0].DrawTiled(Background.ImageSurface,0,0,10,8,11);
  Fonts[0].RenderFont(Background.ImageSurface,170,10,'Treasure Hunter');
  Fonts[0].RenderFont(Background.ImageSurface,150,50,'Light and Darkness');

  Fonts[1].RenderFont(Background.ImageSurface,50,100,'Created for the Ludum Dare ');
  Fonts[1].RenderFont(Background.ImageSurface,50,130,'contest April 16-18th 2005');
  Fonts[1].RenderFont(Background.ImageSurface,50,160,'by');
  Fonts[1].RenderFont(Background.ImageSurface,100,160,'William Cairns');

  Fonts[1].Write(Background.ImageSurface,30,220,'Run through the Maze and ');
  Fonts[1].Write(Background.ImageSurface,30,250,'collect all the treasures that ');
  Fonts[1].Write(Background.ImageSurface,30,280,'you can find. You must also ');
  Fonts[1].Write(Background.ImageSurface,30,310,'collect as many fires as ');
  Fonts[1].Write(Background.ImageSurface,30,340,'possible to keep your torch and  ');
  Fonts[1].Write(Background.ImageSurface,30,370,'vision as large as possible.');
end;

procedure TS2DLGame.SetGameState(const Value: TS2DLGameState);
begin
  If FGameState <> Value then
    Begin
      FGameState := Value;
      Case FGameState of
        gsGameOver:  PlayTheSound(SndGetReady);// mmSystem.PlaySound(PChar('sounds\Laugh.wav'),0,SND_ASYNC); //PlaySound('Laugh');
      End;
      Case FGameState of
        gsTitleProcess: CreateTitleBackGround;
        gsWinning: CreateWinningBackGround;
        gsHelp: Begin
                  HelpPage := 1;
                  CreateHelpBackGround;
                End;
        gsLevelUp: Begin
                     CreateLevelUpBackground;
                     Sleep(100);
                     PlayTheSound(SndGetReady); //PlaySound('GetReady');
                   End;
      Else
        CreateBackground;
      End;
    End;
  LastTick := SDL_GetTicks;
end;

procedure TS2DLGame.LoadMap;
Var
  SL : TStringList;
  I,J : Integer;
  S,S1  :String;
begin
  SL := TStringList.Create;
  Map.DotCount := 0;
  Map.TreasureCount := 0;
  SL.LoadFromFile('data\'+MapName+'.txt');
  For I := 0 to 99 do
    Begin
      S := SL[I];
      J := 0;
      While Length(S) > 0 do
        Begin
          S1 := Copy(S,1,Pos(',',S)-1);
          S := Copy(S,Pos(',',S)+1,Length(S));
          S1 := Copy(S1,Pos('.',S1)+1,Length(S1));
          If S1[1] in ['0'..'9'] then
            Begin
              Map.Map[J,I].Open := StrToInt(S1);
              If (I + J) mod 5 = 0 then
                Begin
                  Map.Map[J,I].Dot := 2;
                  Map.TreasureCount := Map.TreasureCount + 1;
                End
              Else
                Begin
                  Map.Map[J,I].Dot := 0;
                  Map.DotCount := Map.DotCount + 1;
                End;
            End
          Else
            Begin
              Map.Map[J,I].Open := -1;
              Map.Map[J,I].Dot := 1;
            End;
          J := J + 1;
        End;
    End;
  Map.MapName := SL.Values['MapName'];
  Map.Tiles1 := SL.Values['Tiles1'];
  Map.Tiles2 := SL.Values['Tiles2'];
  Map.Tiles3 := SL.Values['Tiles3'];
  Map.NextLevel := SL.Values['NextLevel'];
  Map.Start :=SL.Values['Start'];
  Map.MonsterCount := StrToInt(SL.Values['MonsterCount']);
  For I := 1 to Map.MonsterCount do
    Map.MonsterStart[I] := SL.Values['Monster'+IntToStr(I)];
  Player.Dead := False;
  Player.X := 150;
  Player.Y := 150;
  Keys[1] := False; Keys[2] := False; Keys[3] := False; Keys[4] := False;
end;

procedure TS2DLGame.LevelUpProcess;
begin
  If SDL_GetTicks > LastTick + 3000 then
    Begin
      GameState := gsLevelSetup;
    End;
  Background.Draw(0,0);
  Fonts[0].Write(320-(Length(Map.MapName)*10),80,Map.MapName);
  Fonts[0].Write(240,30,'Level: '+IntToStr(Level));
  Fonts[2].RenderFont(S2DLDraw.Surface,0,0,'FPS: '+IntToStr(S2DLDraw.FPS));
  S2DLDraw.Flip;
end;

procedure TS2DLGame.LevelSetupProcess;
begin
  Player.LastTick := SDL_GetTicks;
  Map.Monsters[1].LastTick := SDL_GetTicks;
  GameState := gsGameProcess;
end;

procedure TS2DLGame.PlayerDiedProcess;
begin
  If SDL_GetTicks > LastTick + 1000 then
    Begin
      GameState := gsGameProcess;
    End;
  Background.Draw(0,0);
  Images.Find('Dead').Draw(100,200);
  Fonts[0].Write(100,100,'Player Died');
  Fonts[2].RenderFont(S2DLDraw.Surface,0,0,'FPS: '+IntToStr(S2DLDraw.FPS));
  S2DLDraw.Flip;
end;

procedure TS2DLGame.PlayerDied(Sender: TObject);
begin
  GameState := gsPlayerDied;
end;

procedure TS2DLGame.GameOverProcess;
begin
  If SDL_GetTicks > LastTick + 3000 then
    Begin
      GameState := gsTitleProcess;
    End;
  Background.Draw(0,0);
  Fonts[0].Write(200,200,'Game Over');
  Fonts[2].RenderFont(S2DLDraw.Surface,0,0,'FPS: '+IntToStr(S2DLDraw.FPS));
  S2DLDraw.Flip;
end;

procedure TS2DLGame.HelpProcess;
Var
  X,Y : Integer;
begin
  { Draw title screen }
  repeat
    while SDL_PollEvent(@event) > 0 do
    begin
      case event.type_ of
        SDL_QUITEV :
           Begin
             Log.LogError('Quit Event','TitleProcess');
             GameState := gsFinish;
           End;
        SDL_MOUSEBUTTONUP:
        Begin
          X := Event.button.X;
          Y := Event.button.Y;
          If (Y > 415) and (Y < 455) then
            Begin
              If (X < 230) and (HelpPage > 1) then
                HelpPage := HelpPage - 1;
              If (X < 440) and (X > 230) then
                GameState := gsTitleProcess;
              If (X > 440) and (HelpPage < 3)  then
                HelpPage := HelpPage + 1;
            End;
        End;

        SDL_KEYDOWN :
        begin
          case Event.Key.keysym.sym of
            SDLK_SPACE :
              GameState := gsGameSetup;
            SDLK_ESCAPE :
              Begin
                GameState := gsFinish;
                Log.LogError('Quit Event','TitleProcess');
              End;
          end;
        end;
      end;
    end;
    // Do screen animations and Game Logic for Title Screen here
    RenderHelp;

    S2DLDraw.Flip;
  until GameState <> gsHelp;
end;

procedure TS2DLGame.RenderHelp;
begin
  Background.Draw(0,0);
  Fonts[2].RenderFont(S2DLDraw.Surface,0,0,'FPS: '+IntToStr(S2DLDraw.FPS));
  If HelpPage > 1 then
    Fonts[0].Write(60,420,'[ Prev ]');
  Fonts[0].Write(270,420,'[ Main ]');
  If HelpPage < 3 then
    Fonts[0].Write(480,420,'[ Next ]');
end;

procedure TS2DLGame.WinningProcess;
Var
  X,Y : Integer;
begin
    while SDL_PollEvent(@event) > 0 do
    begin
      case event.type_ of
        SDL_QUITEV :
          Begin
            Log.LogError('Quit Event','GameProcess');
            GameState := gsFinish;
          End;
        SDL_MOUSEBUTTONUP:
        Begin
          X := Event.button.X;
          Y := Event.button.Y;
          If (Y > 415) and (Y < 455) then
            Begin
              If (X < 440) and (X > 230) then
                GameState := gsTitleProcess;
            End;
        End;
        SDL_KEYUP:
        begin
          case Event.Key.keysym.sym of
            SDLK_ESCAPE: GameState := gsTitleProcess;
            SDLK_SPACE: GameState := gsTitleProcess;
          end;
        end;
      end;
    end;
  Background.Draw(0,0);
  Fonts[2].RenderFont(S2DLDraw.Surface,0,0,'FPS: '+IntToStr(S2DLDraw.FPS));
  S2DLDraw.Flip;
end;

procedure TS2DLGame.CreateWinningBackGround;
begin
  Images[0].DrawTiled(Background.ImageSurface,0,0,10,8,11);
  Fonts[1].Write(Background.ImageSurface,150,30,'Congratulations');
  Fonts[0].Write(Background.ImageSurface,100,90,'!! YOU ARE THE WINNER !!');
  Fonts[1].Write(Background.ImageSurface,50,180,'You have completed all the');
  Fonts[1].Write(Background.ImageSurface,50,210,'levels.');
  Fonts[1].Write(Background.ImageSurface,50,260,'Please send a mail to ');
  Fonts[0].Write(Background.ImageSurface,90,300,'william@cairnsgames.co.za');
  Fonts[1].Write(Background.ImageSurface,50,340,'and let William know.');
  Fonts[0].Write(Background.ImageSurface,270,420,'[ Main ]');
end;

procedure TS2DLGame.CreateHelpBackGround;
begin
  Images[0].DrawTiled(Background.ImageSurface,0,0,10,8,11);
  Case HelpPage of
  1: Begin
       Fonts[0].Write(Background.ImageSurface,250,20,'Goals');
       Fonts[1].Write(Background.ImageSurface,30,100,'Run through the Maze and ');
       Fonts[1].Write(Background.ImageSurface,30,130,'collect all the treasures that ');
       Fonts[1].Write(Background.ImageSurface,30,160,'you can find. You must also ');
       Fonts[1].Write(Background.ImageSurface,30,190,'collect as many fires as ');
       Fonts[1].Write(Background.ImageSurface,30,220,'possible to keep your torch and  ');
       Fonts[1].Write(Background.ImageSurface,30,250,'vision as large as possible.');
     End;
  2: Begin
       Fonts[0].Write(Background.ImageSurface,250,20,'Playing');
       Images.Find('Man').Draw(Background.ImageSurface,50,90,21);
       Fonts[1].Write(Background.ImageSurface,120,100,'Use the arrow keys to  ');
       Fonts[1].Write(Background.ImageSurface,120,130,'move your character ');
       Fonts[1].Write(Background.ImageSurface,30,160,'through the Maze. ');
       Fonts[1].Write(Background.ImageSurface,30,220,'Avoid the Enemies. Some enemies');
       Fonts[1].Write(Background.ImageSurface,30,250,'stand still until you are near');
       Fonts[1].Write(Background.ImageSurface,30,280,'by. Other enemies like patroling ');
       Fonts[1].Write(Background.ImageSurface,30,310,'corridors. The cleverest enemies');
       Fonts[1].Write(Background.ImageSurface,30,340,'come and hunt you, they can see');
       Fonts[1].Write(Background.ImageSurface,30,370,'you anywhere in the Maze. ');
     End;
  3: Begin
       Fonts[0].Write(Background.ImageSurface,250,20,'Enemies');
       Fonts[1].Write(Background.ImageSurface,15,70,'These are the Normal behaviours:');
       Images.Find('Green').Draw(Background.ImageSurface,50,100,21);
         Fonts[1].Write(Background.ImageSurface,120,110,'Patrols');
       Images.Find('Blue').Draw(Background.ImageSurface,50,140,20);
         Fonts[1].Write(Background.ImageSurface,120,150,'Patrols');
       Images.Find('Purple').Draw(Background.ImageSurface,50,190,21);
         Fonts[1].Write(Background.ImageSurface,120,190,'Waits until in view ');
         Fonts[1].Write(Background.ImageSurface,120,215,'then Hunts');
       Images.Find('Mummy').Draw(Background.ImageSurface,50,240,20);
         Fonts[1].Write(Background.ImageSurface,120,250,'Chases while in View');
       Images.Find('Skeleton').Draw(Background.ImageSurface,50,290,20);
         Fonts[1].Write(Background.ImageSurface,120,300,'Chases while in View');
       Images.Find('Lady').Draw(Background.ImageSurface,50,340,20);
         Fonts[1].Write(Background.ImageSurface,120,350,'Hunts');
     End;
  End;
end;

procedure TS2DLGame.SetHelpPage(const Value: Integer);
begin
  FHelpPage := Value;
  CreateHelpBackGround;
end;

procedure TS2DLGame.CreateLevelUpBackground;
Var
  I, I2 : TS2DLImage;
  R : Integer;
  E,M : Integer;
begin
  R := Trunc(Random * 60);
  Images[0].DrawTiled(Background.ImageSurface,0,0,10,8,11);
  I := TS2DLImage.Create;
  I2 := TS2DLImage.Create;
  Case R of
  0..40: Begin
           Case Trunc(Random * 6) of
           0: E := 20;
           1: E := 10;
           2: E := 19;
           3: E := 3;
           4: E := 12;
           Else E := 16;
           End;
           If Random < 0.5 then
             M := 4
           Else
             M := 8;
         End;
  41..59: Begin
            Case Trunc(Random * 100) of
              0..49: E := 18;
              Else E := 15;
            End;
            M := -1;
          End;
  Else Begin
           Case Trunc(Random * 5) of
           0: E := 1;
           1: E := 2;
           2: E := 14;
           3: E := 17;
           Else E := 11;
           End;
           Case Trunc(Random * 4) of
           0: M := 9;
           1: M := 5;
           2: M := 6;
           Else M := 8;
           End;
         End;
  End;
  If E = 1 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme1.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,320,100);
    End;
  If E = 2 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme2.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,340,120);
    End;
  If E = 3 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme3.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,170,100);
    End;
  If E = 10 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme10.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,130,80);
    End;
  If E = 12 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme12.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,160,100);
    End;
  If E = 13 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme13.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,130,80);
    End;
  If E = 14 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme14.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,330,110);
    End;
  If E = 15 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme15.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,230,80);
    End;
  If E = 16 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme16.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,170,110);
    End;
  If E = 17 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme17.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,320,100);
    End;
  If E = 18 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme18.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,250,130);
    End;
  If E = 19 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme19.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,130,80);
    End;
  If E = 20 then
    Begin
      I.LoadFromFile(PChar(AppPath+'images\Theme20.png'));
      I.TransparentColor(0,0,0);
      I.Draw(Background.ImageSurface,130,100);
    End;
  If M = 4 then
    Begin
      I2.LoadFromFile(PChar(AppPath+'images\Theme4.png'));
      I2.TransparentColor(0,0,0);
      I2.Draw(Background.ImageSurface,230,150);
    End;
  If M = 5 then
    Begin
      I2.LoadFromFile(PChar(AppPath+'images\Theme5.png'));
      I2.TransparentColor(0,0,0);
      I2.Draw(Background.ImageSurface,230,150);
    End;
  If M = 6 then
    Begin
      I2.LoadFromFile(PChar(AppPath+'images\Theme6.png'));
      I2.TransparentColor(0,0,0);
      I2.Draw(Background.ImageSurface,230,130);
    End;
  If M = 8 then
    Begin
      I2.LoadFromFile(PChar(AppPath+'images\Theme8.png'));
      I2.TransparentColor(0,0,0);
      I2.Draw(Background.ImageSurface,260,130);
    End;
  If M = 9 then
    Begin
      I2.LoadFromFile(PChar(AppPath+'images\Theme9.png'));
      I2.TransparentColor(0,0,0);
      I2.Draw(Background.ImageSurface,230,150);
    End;
  I.Free;
  I2.Free;
end;

end.
