unit LD48MonsterUnit;

interface

Uses
  SysUtils, Classes,
  S2DLLogger,
  SDL,
  S2DLImage,
  S2DLDraws,
  S2DLText,
  S2DLAI, dws2Exprs, LD48Path, mmSystem;

Const
  GemSize = 16; GemSizeHalf = 8; GemSizeEdge = 24;
  FireSize = 16; FireSizeHalf = 8; FireSizeEdge = 24;
  TileSize = 64; TileSizeHalf = 32;
  PlayerSize = 32; PlayerSizeHalf = 16; PlayerSizeTile = 16;
  CircleSize = 384{256}; CircleSizeHalf = 192{128};
  MoveSpeed = 0.15;

Var
  Keys : Array[1..4] of Boolean;


Type
  TOnDie = Procedure (Sender : TObject) of Object;
  TPlayer = Class;
  TMonster = Class;
  TMap = Class
    Map : Array[0..99,0..99] of Record
        Open : Integer;
        Dot : Integer;
        Visible : Boolean;
      End;
    DotCount : Integer;
    TreasureCount : Integer;
    MapLeft,MapTop : Integer;
    Player : TPlayer;
    Monsters : Array[1..20] of TMonster;
    MapName : String;
    Tiles1, Tiles2, Tiles3 : String;
    NextLevel : String;
    Start : String;
    MonsterCount : Integer;
    MonsterStart : Array[1..20] of String;
    LightCount : Integer;
    LightBright : Integer;
    OnDie : TOnDie;
    Procedure Draw;
    Procedure Draw2;
    Procedure Move;
    Procedure Init;
  End;
  TCreature = Class
  Private
    FY: Single;
    FX: Single;
    FMap: TMap;
    procedure SetX(const Value: Single); Virtual;
    procedure SetY(const Value: Single); Virtual;
    procedure SetMap(const Value: TMap);
  Public
    MX,MY,DX,DY : Integer;
    LastTick : Cardinal;
    MoveSpeed : Single;
    Dead : Boolean;
    FrameTick : Integer;
    FrameNo : Integer; FrameBase : Integer;
    Property X : Single read FX write SetX;
    Property Y : Single read FY write SetY;
    Property Map : TMap read FMap write SetMap;
    Constructor Create; Virtual;
    Procedure Move; Virtual;
    Procedure Draw; Virtual;
  End;
  TPlayer = Class(TCreature)
    Score : Integer;
    Lives : Integer;
    procedure SetX(const Value: Single); Override;
    procedure SetY(const Value: Single); Override;
    Procedure Move; Override;
    Procedure Draw; Override;
    Procedure Init;
  End;
  TMonster = Class(TCreature)
    AI : TS2DLAI;
    LocalKeys : Array[1..4] of Boolean;
  private
    FScriptName: String;
    FPlayer: TPlayer;
    LastMove : Cardinal;
    procedure SetPlayer(const Value: TPlayer);
    procedure SetScriptName(const Value: String);
    procedure KeyDown(Info : TProgramInfo);
    procedure KeyUp(Info : TProgramInfo);
    Procedure GetPlayer(Info : TProgramInfo);
    Procedure GetSelf(Info : TProgramInfo);
    Procedure TileAt(Info : TProgramInfo);
    Procedure ItemAt(Info : TProgramInfo);
    Procedure CanMoveTo(Info : TProgramInfo);
    Procedure CanSee(Info : TProgramInfo);
    Procedure BestDir(Info : TProgramInfo);
    Procedure ClearKeys(Info : TProgramInfo);
    procedure CreateRecords;
    Procedure CreateFunctions;
  Public
    Image : TS2DLImage;
    Constructor Create; Override;
    Property ScriptName : String read FScriptName write SetScriptName;
    Property Player : TPlayer read FPlayer write SetPlayer;
    Property Map : TMap read FMap write SetMap;
    Procedure LoadScript(ScriptName : String);
    Procedure Start;
    Procedure Execute;
    Procedure Finish;
    Procedure RunProcedure(ProcName : String);
    Procedure Move; Override;
    Procedure Draw; Override;
    Procedure DrawEyes;
  End;

procedure LoadWav(const Filename: string; var Ptr: pointer);
Procedure PlayTheSound(Ptr : Pointer);

Var
  SndGetReady : Pointer;
  sndShot : Pointer;
  sndLaugh : Pointer;

implementation

Procedure PlayTheSound(Ptr : Pointer);
Begin
  mmSystem.PlaySound(PChar(Ptr), 0, SND_MEMORY or SND_NODEFAULT or SND_ASYNC);
End;


procedure LoadWav(const Filename: string; var Ptr: pointer);
  var
    Strm: TFileStream;
    Size: integer;
  begin
    Strm := TFileStream.Create(Filename, fmOpenRead);
    GetMem(Ptr, Strm.Size);
    Strm.ReadBuffer(Ptr^, Strm.Size);
    Strm.Free;
  end;


{ TMap }
Const
TileDirs : Array[0..11,1..4] of Boolean =
  ((False,False,True,True),
   (True,True,True,True),
   (True,True,False,True),
   (False,True,True,False),
   (True,True,False,False),
   (True,True,True,False),
   (False,True,True,True),
   (True,False,True,True),
   (True,False,True,False),
   (False,True,False,True),
   (True,False,False,True),
   (True,True,True,True));

procedure TMap.Draw;
Var
  IX,IY,MinX,MinY : Integer;
  I : Integer;
begin
  MinY := Player.MY - 3; If MinY < 0 then MinY := 0;
  MinX := Player.MX - 3; If MinX < 0 then MinX := 0;
  For IX := MinX to MinX+7 do
    For IY := MinY to MinY+7 do
      Begin
        If Map[IX,IY].Open = -1 then
          Images[0].Draw(IX * TileSize - MapLeft, IY * TileSize - MapTop,12)
        Else
          Images[0].Draw(IX * TileSize - MapLeft, IY * TileSize - MapTop,Map[IX,IY].Open);
//        Fonts[3].Write(IX * TileSize - MapLeft, IY * TileSize - MapTop,IntToStR(LD48Path.Access[IX,IY]));
        If Map[IX,IY].Dot = 0 then
          Images[3].Draw(IX * TileSize - MapLeft + FireSizeEdge, IY * TileSize - MapTop + FireSizeEdge,Trunc(Random*8));
        If Map[IX,IY].Dot = 2 then
          Images[4].Draw(IX * TileSize - MapLeft + GemSizeEdge, IY * TileSize - MapTop + GemSizeEdge,0);
      End;
  Player.Draw;
  For I := 1 to 20 do
    If Monsters[I] <> Nil then
      Monsters[I].Draw;
end;

procedure TMap.Draw2;
Var
  I : Integer;
begin
  For I := 1 to 20 do
    If Monsters[I] <> Nil then
      Monsters[I].DrawEyes;
end;

procedure TMap.Init;
Var
  I : Integer;
  S : String;
begin
  Player.LastTick := SDL_GetTicks;
  For I := 1 to MonsterCount do
    Begin
      If Monsters[I] = Nil then
        Begin
          Monsters[I] := TMonster.Create;
          Monsters[I].Map := Self;
          Monsters[I].LoadScript('data\basic.ai');
          Monsters[I].MoveSpeed := MoveSpeed * 0.75;
        End;
      If Monsters[I] <> Nil then
        Begin
          Monsters[I].LastTick := SDL_GetTicks;
          S := MonsterStart[I];
          Monsters[I].X := StrToInt(Copy(S,1,Pos(',',S)-1)) * 64 + 32;
          S := Copy(S,Pos(',',S)+1,Length(S));
          Monsters[I].Y := StrToInt(Copy(S,1,Pos(',',S)-1)) * 64 + 32;
          S := Copy(S,Pos(',',S)+1,Length(S));
          Monsters[I].Image := Images.Find(Copy(S,1,Pos(',',S)-1));
          S := Copy(S,Pos(',',S)+1,Length(S));
          Monsters[I].LoadScript('data\'+S+'.AI');
          Monsters[I].AI.Init;
        End;
    End;
  Player.X := StrToInt(Copy(Start,1,Pos(',',Start)-1)) * 64 + 32;
  Player.Y := StrToInt(Copy(Start,Pos(',',Start)+1,Length(Start))) * 64 + 32;
  LightCount := 1000;
  LightBright := 0;
end;

procedure TMap.Move;
Var
  X,Y : Single;
  I : Integer;
begin
  For I := 1 to 20 do
    If Monsters[I] <> Nil then
    Begin
      Monsters[I].Move;
      X := ABS(Player.X - Monsters[I].X);
      Y := Player.Y - Monsters[I].Y;
      X := SQRT(X * X + Y * Y);
      If X < 32 then
        Begin
          Player.Lives := Player.Lives - 1;
          OnDie(Self);
          If Player.Lives = 0 then
            Player.Dead := True
          Else
            Begin
              Init;
            End;
        End;
    End;
end;

{ TPlayer }

constructor TCreature.Create;
begin
  FrameTick := SDL_GetTicks;
  FrameNo := 0;
  FrameBase := 0;
end;

procedure TCreature.Draw;
begin
end;

procedure TCreature.Move;
Var
  TC : Cardinal;
Begin
  If FrameBase < 16 then
    Begin
      TC := SDL_GeTTicks - FrameTick;
      If TC > 150 then
        Begin
          FrameNo := FrameNo + 1;
          If FrameNo = 5 then
            FrameNo := 0;
          FrameTick := SDL_GetTicks;
        End;
    End;
end;

procedure TCreature.SetMap(const Value: TMap);
begin
  FMap := Value;
end;

procedure TCreature.SetX(const Value: Single);
begin
  FX := Value;
  If FX < 16 then FX := 16;
  DX := Round(X);
  MX := DX div 64;
end;

procedure TCreature.SetY(const Value: Single);
begin
  FY := Value;
  If FY < 16 then FY := 16;
  DY := Round(Y);
  MY := DY div 64;
end;

{ TPlayer }

procedure TPlayer.Draw;
begin
  inherited;
//  Images[2].Draw(DX-PlayerSizeHalf-Map.MapLeft,DY-PlayerSizeHalf-Map.MapTop,0);
  Images[6].Draw(DX-PlayerSizeHalf-Map.MapLeft-14,DY-PlayerSizeHalf-Map.MapTop-14,FrameBase+FrameNo);
end;

procedure TPlayer.Init;
begin
  Score := 0;
  Lives := 3;
end;

procedure TPlayer.Move;
Var
  TX,TY : Single;
  XX,XY,ZX,ZY,IX,IY : Integer;
  TC : Cardinal;
begin
  inherited;
  TC := SDL_GetTicks - LastTick;
  TX := X; TY := Y; IX := MX; IY := MY;
  If Keys[1] then Y := Y - MoveSpeed * TC;
  If Keys[2] then Y := Y + MoveSpeed * TC;
  If Keys[3] then X := X - MoveSpeed * TC;
  If Keys[4] then X := X + MoveSpeed * TC;
  If Keys[1] then FrameBase := 0;
  If Keys[2] then FrameBase := 5;
  If Keys[3] then FrameBase := 15;
  If Keys[4] then FrameBase := 10;
  If Not(Keys[1] or Keys[2] or Keys[3] or Keys[4]) then
    FrameBase := 16;
  ZX := (DX - PlayerSizeTile) div TileSize; XX := (DX + PlayerSizeTile) div TileSize;
  If ((ZX < IX) and (Not TileDirs[Map.Map[IX,IY].Open,3]) and (Keys[3])) or
     ((XX > IX) and (Not TileDirs[Map.Map[IX,IY].Open,4]) and (Keys[4])) then
    X := TX;
  ZY := (DY - PlayerSizeTile) div TileSize; XY := (DY + PlayerSizeTile) div TileSize;
  If ((ZY < IY) and (Not TileDirs[Map.Map[IX,IY].Open,1]) and (Keys[1])) or
     ((XY > IY) and (Not TileDirs[Map.Map[IX,IY].Open,2]) and (Keys[2])) then
    Y := TY;
  LastTick := SDL_GetTicks;
end;

procedure TPlayer.SetX(const Value: Single);
begin
  inherited;
  Map.MapLeft := DX - 304;

end;

procedure TPlayer.SetY(const Value: Single);
begin
  inherited;
  Map.MapTop := DY - 224;

end;

{ TMonster }

procedure TMonster.BestDir(Info: TProgramInfo);
Var
  FromX,FromY,ToX,ToY,Dir : Integer;
  I,J : Integer;
begin
  FromX := Info.Value['FromX'];
  FromY := Info.Value['FromY'];
  ToX := Info.Value['ToX'];
  ToY := Info.Value['ToY'];
  If (FromX = ToX) and (FromY = ToY) then
    Begin
      Info.Result := 0;
      Exit;
    End;
  // Return the best direction to travel in to reach the player
  InitMap;
  For I := 0 to 99 do
    For J := 0 to 99 do
      Begin
        LD48Path.Map[I,J] := Map.Map[I,J].Open;
      End;
  Info.Result := LD48Path.BestDir(FromX,FromY,ToX,ToY);
end;

procedure TMonster.CanMoveTo(Info: TProgramInfo);
Var
  FromX,FromY,ToX,ToY,Dir : Integer;
begin
  FromX := Info.Value['FromX'];
  FromY := Info.Value['FromY'];
  ToX := Info.Value['ToX'];
  ToY := Info.Value['ToY'];
  If ((ToX-FromX) = -1) and ((ToY-FromY) = 0) then Dir := 3;
  If ((ToX-FromX) = 1) and ((ToY-FromY) = 0) then Dir := 4;
  If ((ToX-FromX) = 0) and ((ToY-FromY) = -1) then Dir := 1;
  If ((ToX-FromX) = 0) and ((ToY-FromY) = 1) then Dir := 2;
  If TileDirs[Map.Map[FromX,FromY].Open,Dir] then
    Info.Result := 1
  Else
    Info.Result := -1;
end;

procedure TMonster.CanSee(Info: TProgramInfo);
Var
  FromX,FromY,ToX,ToY,Dir : Integer;
begin
  // If player closer than 3 squares away. - Any Direction
  FromX := Info.Value['FromX'];
  FromY := Info.Value['FromY'];
  ToX := Info.Value['ToX'];
  ToY := Info.Value['ToY'];
  If (Abs(FromX - ToX) <= 2) and
     (Abs(FromY - ToY) <= 2) then
    Info.Result := True
  Else
    Info.Result := False;
end;

procedure TMonster.ClearKeys(Info: TProgramInfo);
begin
  LocalKeys[1] := False;
  LocalKeys[2] := False;
  LocalKeys[3] := False;
  LocalKeys[4] := False;
end;

constructor TMonster.Create;
begin
  Inherited;
  AI := TS2DLAI.Create;
  LastMove := SDL_GetTicks;
  CreateRecords;
  CreateFunctions;
end;

procedure TMonster.CreateFunctions;
begin
  AI.CreateProcedure('KeyUp',['ID:Integer'],KeyUp);
  AI.CreateProcedure('KeyDown',['ID:Integer'],KeyDown);
  AI.CreateProcedure('ClearKeys',[],ClearKeys);
  AI.CreateProcedure('GetPlayer',['ID:Integer','var PlayerInfo:TPlayerInfo'],GetPlayer);
  AI.CreateProcedure('GetSelf',['var PlayerInfo:TPlayerInfo'],GetSelf);
  AI.CreateFunction('TileAt',['TileX:Integer','TileY:Integer'],TileAt,'Integer');
  AI.CreateFunction('ItemAt',['TileX:Integer','TileY:Integer'],ItemAt,'Integer');
  AI.CreateFunction('CanMoveTo',['FromX:Integer','FromY:Integer','ToX:Integer','ToY:Integer'],CanMoveTo,'Integer');
  AI.CreateFunction('CanSee',['FromX:Integer','FromY:Integer','ToX:Integer','ToY:Integer'],CanSee,'Boolean');
  AI.CreateFunction('BestDir',['FromX:Integer','FromY:Integer','ToX:Integer','ToY:Integer'],BestDir,'Integer');
end;

procedure TMonster.CreateRecords;
begin
  AI.CreateRecord('TPlayerInfo',['Name:String','X:Integer','Y:Integer',
                                  'CX:Integer','CY:Integer']);
end;

procedure TMonster.Draw;
begin
  inherited;
//  Images[5].Draw(DX-PlayerSizeHalf-Map.MapLeft,DY-PlayerSizeHalf-Map.MapTop,0);
  Image.Draw(DX-PlayerSizeHalf-Map.MapLeft-14,DY-PlayerSizeHalf-Map.MapTop-14,FrameBase+FrameNo);
end;

procedure TMonster.DrawEyes;
Var
  FromX,ToX,FromY,ToY : Integer;
begin
  FromX := Map.Player.MX;
  FromY := Map.Player.MY;
  ToX := MX;
  ToY := MY;
  If Not((Abs(FromX - ToX) <= 2) and
     (Abs(FromY - ToY) <= 2)) then
    Images[5].Draw(DX-PlayerSizeHalf-Map.MapLeft,DY-PlayerSizeHalf-Map.MapTop,1);
end;

procedure TMonster.Execute;
begin
  AI.Execute;
end;

procedure TMonster.Finish;
begin
  AI.Finish;

end;

procedure TMonster.GetPlayer(Info: TProgramInfo);
begin
  Info.Vars['PlayerInfo'].Member['Name'].Value := 'Player';
  Info.Vars['PlayerInfo'].Member['X'].Value := Map.Player.DX;
  Info.Vars['PlayerInfo'].Member['Y'].Value := Map.Player.DY;
  Info.Vars['PlayerInfo'].Member['CX'].Value := Map.Player.DX div TileSize;
  Info.Vars['PlayerInfo'].Member['CY'].Value := Map.Player.DY div TileSize;
end;

procedure TMonster.GetSelf(Info: TProgramInfo);
begin
  Info.Vars['PlayerInfo'].Member['Name'].Value := 'Monster';
  Info.Vars['PlayerInfo'].Member['X'].Value := DX;
  Info.Vars['PlayerInfo'].Member['Y'].Value := DY;
  Info.Vars['PlayerInfo'].Member['CX'].Value := DX div TileSize;
  Info.Vars['PlayerInfo'].Member['CY'].Value := DY div TileSize;
end;

procedure TMonster.ItemAt(Info: TProgramInfo);
Var
  X,Y : Integer;
begin
  X := Info.Value['TileX'];
  Y := Info.Value['TileY'];
  Info.Result := Map.Map[X,Y].Dot;
end;

procedure TMonster.KeyDown(Info: TProgramInfo);
Var
  ID : Integer;
Begin
  ID := Info.Value['ID'];
  If ID in [1..4] then
    LocalKeys[ID] := True;
end;

procedure TMonster.KeyUp(Info: TProgramInfo);
Var
  ID : Integer;
Begin
  ID := Info.Value['ID'];
  If ID in [1..4] then
    LocalKeys[ID] := False;
end;

procedure TMonster.LoadScript(ScriptName: String);
Var
  Script : TStringList;
  S : String;
begin
  Script := TStringList.Create;
  Script.LoadfromFile(ScriptName);
  S := Script.Text;
  AI.AIScript.Text := Script.Text;
  Script.Free;
  AI.Compile;
  Start;
end;

procedure TMonster.Move;
Var
  TX,TY : Single;
  XX,XY,ZX,ZY,IX,IY : Integer;
  TC : Cardinal;
begin
  inherited;
  If SDL_GetTicks - LastMove > 100 then
    Begin
      AI.Execute;
      LastMove := SDL_GetTicks;
    End;
  TC := SDL_GetTicks - LastTick;
  If TC > 100 then TC := 100;
  TX := X; TY := Y; IX := MX; IY := MY;
  If LocalKeys[1] then Y := Y - MoveSpeed * TC;
  If LocalKeys[2] then Y := Y + MoveSpeed * TC;
  If LocalKeys[3] then X := X - MoveSpeed * TC;
  If LocalKeys[4] then X := X + MoveSpeed * TC;
  If LocalKeys[1] then FrameBase := 0;
  If LocalKeys[2] then FrameBase := 5;
  If LocalKeys[3] then FrameBase := 15;
  If LocalKeys[4] then FrameBase := 10;
  If Not(LocalKeys[1] or LocalKeys[2] or LocalKeys[3] or LocalKeys[4]) then
    FrameBase := 16;
  ZX := (DX - PlayerSizeTile) div TileSize; XX := (DX + PlayerSizeTile) div TileSize;
  If ((ZX < IX) and (Not TileDirs[Map.Map[IX,IY].Open,3]) and (LocalKeys[3])) or
     ((XX > IX) and (Not TileDirs[Map.Map[IX,IY].Open,4]) and (LocalKeys[4])) then
    Begin
      X := TX;
      AI.RunProcedure('CantMove');
    End;
  ZY := (DY - PlayerSizeTile) div TileSize; XY := (DY + PlayerSizeTile) div TileSize;
  If ((ZY < IY) and (Not TileDirs[Map.Map[IX,IY].Open,1]) and (LocalKeys[1])) or
     ((XY > IY) and (Not TileDirs[Map.Map[IX,IY].Open,2]) and (LocalKeys[2])) then
    Begin 
      Y := TY;
      AI.RunProcedure('CantMove');
    End;
  LastTick := SDL_GetTicks;
end;

procedure TMonster.RunProcedure(ProcName: String);
begin
  AI.RunProcedure(ProcName);
end;

procedure TMonster.SetPlayer(const Value: TPlayer);
begin
  FPlayer := Value;
end;

procedure TMonster.SetScriptName(const Value: String);
begin
  FScriptName := Value;
end;

procedure TMonster.Start;
begin
  AI.RunProcedure('Init');
end;

procedure TMonster.TileAt(Info: TProgramInfo);
Var
  X,Y : Integer;
begin
  X := Info.Value['TileX'];
  Y := Info.Value['TileY'];
  Info.Result := Map.Map[X,Y].Open

end;

end.
