unit S2DLAI;

interface

Uses
  Windows, SysUtils, Classes, dws2Comp, dws2Exprs;

Type
  TS2DLAI = Class
  Private
    Script: TDelphiWebScriptII;
    dws2Unit: Tdws2Unit;
    FAIScript: TStringList;
    FMain: String;
    procedure SetAIScript(const Value: TStringList);
    procedure SetMain(const Value: String);
    Procedure GetTickCount(Info : TProgramInfo);
  Public
    FProg: TProgram;
    Property AIScript : TStringList read FAIScript write SetAIScript;
    Property Main : String read FMain write SetMain;
    Constructor Create; Virtual;
    Procedure CreateFunction(Name : String; Params : Array of String; OnEval : TFuncEvalEvent; ResultType : String);
    Procedure CreateRecord(Name : String; Params : Array of String);
    Procedure CreateProcedure(Name : String; Params : Array of String; OnEval : TFuncEvalEvent);
    Procedure Compile;
    Procedure Init;
    Procedure Execute;
    Procedure Finish;
    Procedure RunProcedure(ProcName : String);
  End;

implementation

{ TS2DLAI }

procedure TS2DLAI.Compile;
begin
  Try
    FProg := Script.Compile(AIScript.Text);
  Except
    On Exception do
      Begin
       // If error on script then load default script and recompile
      End;
  End;
  FProg.BeginProgram;
end;

constructor TS2DLAI.Create;
begin
  Main := 'Execute';
  FAIScript := TStringList.Create;
  dws2Unit := Tdws2Unit.Create(Nil);
  dws2Unit.UnitName := 'OutputFuncs';
  Script := TDelphiWebScriptII.Create(Nil);
  dws2Unit.Script := Script;
  CreateFunction('GetTickCount',[],GetTickCount,'Integer');
end;

procedure TS2DLAI.CreateFunction(Name: String; Params: array of String;
  OnEval: TFuncEvalEvent; ResultType : String);
Var
  F : Tdws2Function;
  P : TDWS2Parameter;
  I : Integer;
begin
  F := Tdws2Function(Dws2Unit.Functions.Add);
  F.Name := Name;
  F.OnEval := OnEval;
  F.ResultType := ResultType;
  For I := 0 to High(Params) do
    Begin
      P := TDWS2Parameter(F.Parameters.Add);
      P.Name := Copy(Params[I],1,Pos(':',Params[I])-1);
      P.DataType := Copy(Params[I],Pos(':',Params[I])+1,Length(Params[I]));
    End;
end;

procedure TS2DLAI.CreateProcedure(Name: String; Params: array of String;
  OnEval: TFuncEvalEvent);
Var
  F : Tdws2Function;
  P : TDWS2Parameter;
  I : Integer;
  S : String;
begin
  If Dws2Unit.Functions.IndexOf(Name) > -1 then Exit;
  F := Tdws2Function(Dws2Unit.Functions.Add);
  F.Name := Name;
  F.OnEval := OnEval;
  For I := 0 to High(Params) do
    Begin
      P := TDWS2Parameter(F.Parameters.Add);
      S := Params[I];
      If Pos('var ',S)>0 then
        Begin
          S := Copy(S,Pos(' ',S)+1,Length(S));
          P.IsVarParam := True;
        End;
      P.Name := Copy(S,1,Pos(':',S)-1);
      P.DataType := Copy(S,Pos(':',S)+1,Length(S));
    End;
end;

procedure TS2DLAI.CreateRecord(Name: String; Params: array of String);
Var
  R : Tdws2Record;
  M : TDWS2Member;
  I : Integer;
begin
  If Dws2Unit.Records.IndexOf(Name) > -1 then Exit;
  R := Tdws2Record(Dws2Unit.Records.Add);
  R.Name := Name;
  For I := 0 to High(Params) do
    Begin
      M := TDWS2Member(R.Members.Add);
      M.Name := Copy(Params[I],1,Pos(':',Params[I])-1);
      M.DataType := Copy(Params[I],Pos(':',Params[I])+1,Length(Params[I]));
      M.Name := Copy(Params[I],1,Pos(':',Params[I])-1);
    End;
end;

procedure TS2DLAI.Execute;
begin
  FProg.Info.Func[Main].Call;
end;

procedure TS2DLAI.Finish;
begin
  FProg.EndProgram;
end;

procedure TS2DLAI.GetTickCount(Info: TProgramInfo);
begin
  Info.Result := Windows.GetTickCount;
end;

procedure TS2DLAI.Init;
begin
  RunProcedure('Init');
end;

procedure TS2DLAI.RunProcedure(ProcName: String);
begin
  FProg.Info.Func[ProcName].Call
end;

procedure TS2DLAI.SetAIScript(const Value: TStringList);
begin
  FAIScript := Value;
end;

procedure TS2DLAI.SetMain(const Value: String);
begin
  FMain := Value;
end;

end.
