unit S2DLLogger;

{$ifdef FPC}
{$mode delphi}
{$endif}

Interface

//  Simple 2D Library - Program Error/Status Logging Unit
//          version 1.04
//
//  This file is part of the Free Pascal Simple 2D Libraries
//    Copyright (c) 2004 by William Cairns (CairnsGames) and
//                         the Delphi Game Development (DGDev) Site
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//       (I got this from the JEDI-SDL files and it sounds Cool!)
//
// By using this work you have immediatly agreed with the above.
//
// The work can be used for any purpose. You may sell it, use it in a game
// delete it or change it as you wish with no reward the the original developers.
//
// We do however ask that if this work is used that some emntion of it be made
// on the DGDev or PGDev web sites.
//
//          Based on the Error Logging Unit released with JEDI-SDL
//         Full copyright code for original logger included below
//
// Changes for S2DLLogger unit
//   Set bedug level to control level of log detail
//   Auto check config file for debug level
//      If no config file found then uses highest level
//    0 = No Logging
//    1 = Errors only
//    2 = Errors and Warnings
//    3 = Errors, Warning and Status messages = All
//
//               Error Logging Unit
//
//The initial developer of the JEDI-SDL Logger Pascal code was :                              }
//Dominique Louis <Dominique@SavageSoftware.com.au>
//
//Portions created by Dominique Louis are
//Copyright (C) 2000 - 2001 Dominique Louis.
//
//Obtained through:
//Joint Endeavour of Delphi Innovators ( Project JEDI )
//

uses
  Classes,
  SysUtils;
  
type
  TLogger = class(TFileStream)
  private
    FApplicationName : string;
    FApplicationPath : string;
    FLogLevel : Integer;
    procedure SetLogLevel(const AValue : Integer);
  protected

  public
    constructor Create;
    procedure LogError( ErrorMessage : string; Location : string );
    procedure LogWarning( WarningMessage : string; Location : string );
    procedure LogStatus( StatusMessage : string; Location : string );
    procedure CreateFile;
  published
    property ApplicationName : string read FApplicationName;
    property ApplicationPath : string read FApplicationPath;
    property LogLevel : Integer read FLogLevel write SetLogLevel;
  end;

var
  Log : TLogger;

implementation


{ TLogger }
constructor TLogger.Create;
Var
  L : TStringList;
  V : String;
begin
  FApplicationName := ExtractFileName( ParamStr(0) );
  FApplicationPath := ExtractFilePath( ParamStr(0) );
  LogLevel := 0;
  If FileExists(FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.cfg') then
    begin
      L := TStringList.Create;
      L.LoadFromFile(FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.cfg');
      V := L.Values['Debug'];
      If V = '' then
        V := '3';
      If Not (V[1] in ['0','1'..'3']) then
        V := '3';
      LogLevel := StrToInt(V);
      L.Free;
    End
  Else
    LogLevel := 3;
end;

procedure TLogger.SetLogLevel(const AValue : Integer);
begin
  if FLogLevel=AValue then exit;
  If FLogLevel = 0 then
    CreateFile;
  FLogLevel:=AValue;
end;

procedure TLogger.CreateFile;
begin
  inherited Create( FApplicationPath + Copy( FApplicationName, 0, Length( FApplicationName ){$IFDEF WIN32} - 4{$ENDIF} ) + '.log',
                    fmCreate {$IFNDEF FPC}or fmShareExclusive{$ENDIF} );
end;

procedure TLogger.LogError(ErrorMessage, Location: string);
var
  S : string;
begin
  If LogLevel > 0 then
    Begin
      S := '*** ERROR *** : @ ' + TimeToStr(Time) + ' MSG : ' + ErrorMessage + ' IN : ' + Location + #13#10;
      Write(S[1], Length(S));
    End;
end;

procedure TLogger.LogStatus(StatusMessage, Location: string);
var
  S : string;
begin
  If LogLevel > 2 then
    Begin
      S := 'STATUS INFO : @ ' + TimeToStr(Time) + ' MSG : ' + StatusMessage + ' IN : ' + Location + #13#10;
      Write(S[1], Length(S));
    End;
end;

procedure TLogger.LogWarning(WarningMessage, Location: string);
var
  S : string;
begin
  If LogLevel > 1 then
    Begin
      S := '=== WARNING === : @ ' + TimeToStr(Time) + ' MSG : ' + WarningMessage + ' IN : ' + Location + #13#10;
      Write(S[1], Length(S));
    End;
end;

initialization
begin
  Log := TLogger.Create;
  Log.LogStatus( 'Starting Application', 'Initialization' );
end;

finalization
begin
  Log.LogStatus( 'Terminating Application', 'Finalization' );
  Log.Free;
  Log := nil;
end;

end.
 
